@@ -50,19 +50,17 @@ def run_linters(files):
     run_detekt(files)
 
 
-def test_parser(tmpdir):
+def test_parser(tmp_path):
     """Test translating metrics to Kotlin files."""
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "core.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo"},
         {"allow_reserved": True},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         [
             "CorePing.kt",
             "Telemetry.kt",
@@ -74,68 +72,64 @@ def test_parser(tmpdir):
     )
 
     # Make sure descriptions made it in
-    with (tmpdir / "CorePing.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "CorePing.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert "True if the user has set Firefox as the default browser." in content
         # Make sure the namespace option is in effect
         assert "package Foo" in content
 
-    with (tmpdir / "Telemetry.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Telemetry.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert "جمع 搜集" in content
 
-    with (tmpdir / "GleanInternalMetrics.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "GleanInternalMetrics.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert 'category = ""' in content
 
-    with (tmpdir / "GleanBuildInfo.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "GleanBuildInfo.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert "buildDate = Calendar.getInstance" in content
 
-    run_linters(tmpdir.glob("*.kt"))
+    run_linters(tmp_path.glob("*.kt"))
 
 
-def test_parser_all_metrics(tmpdir):
+def test_parser_all_metrics(tmp_path):
     """Test translating ALL metric types to Kotlin files."""
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "all_metrics.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo", "with_buildinfo": "false"},
         {"allow_reserved": False},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(["AllMetrics.kt"])
+    assert set(x.name for x in tmp_path.iterdir()) == set(["AllMetrics.kt"])
 
-    run_linters(tmpdir.glob("*.kt"))
+    run_linters(tmp_path.glob("*.kt"))
 
 
-def test_ping_parser(tmpdir):
+def test_ping_parser(tmp_path):
     """Test translating pings to Kotlin files."""
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "pings.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo"},
         {"allow_reserved": True},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Pings.kt", "GleanBuildInfo.kt"]
     )
 
     # Make sure descriptions made it in
-    with (tmpdir / "Pings.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Pings.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert "This is a custom ping" in content
         # Make sure the namespace option is in effect
         assert "package Foo" in content
 
-    run_linters(tmpdir.glob("*.kt"))
+    run_linters(tmp_path.glob("*.kt"))
 
 
 def test_kotlin_generator():
@@ -207,25 +201,25 @@ def test_metric_type_name():
     assert kotlin.type_name(ping) == "PingType<customReasonCodes>"
 
 
-def test_duplicate(tmpdir):
+def test_duplicate(tmp_path):
     """
     Test that there aren't duplicate imports when using a labeled and
     non-labeled version of the same metric.
 
     https://github.com/mozilla-mobile/android-components/issues/2793
     """
-
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
-        ROOT / "data" / "duplicate_labeled.yaml", "kotlin", tmpdir, {"namespace": "Foo"}
+        ROOT / "data" / "duplicate_labeled.yaml",
+        "kotlin",
+        tmp_path,
+        {"namespace": "Foo"},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Category.kt", "GleanBuildInfo.kt"]
     )
 
-    with (tmpdir / "Category.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Category.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert (
             content.count(
@@ -235,202 +229,99 @@ def test_duplicate(tmpdir):
         )
 
 
-def test_glean_namespace(tmpdir):
+def test_glean_namespace(tmp_path):
     """
     Test that setting the glean namespace works.
     """
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "duplicate_labeled.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo", "glean_namespace": "Bar"},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Category.kt", "GleanBuildInfo.kt"]
     )
 
-    with (tmpdir / "Category.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Category.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         assert content.count("import Bar.private.CounterMetricType") == 1
 
 
-def test_gecko_datapoints(tmpdir):
-    """Test translating metrics to Kotlin files."""
-    tmpdir = Path(str(tmpdir))
-
-    translate.translate(
-        ROOT / "data" / "gecko.yaml",
-        "kotlin",
-        tmpdir,
-        {"glean_namespace": "Bar"},
-        {"allow_reserved": True},
-    )
-
-    metrics_files = [
-        "GfxContentCheckerboard.kt",
-        "GfxInfoAdapter.kt",
-        "PagePerf.kt",
-        "NonGeckoMetrics.kt",
-        "GleanBuildInfo.kt",
-    ]
-    assert set(x.name for x in tmpdir.iterdir()) == set(
-        ["GleanGeckoMetricsMapping.kt"] + metrics_files
-    )
-
-    # Make sure descriptions made it in
-    with (tmpdir / "GleanGeckoMetricsMapping.kt").open("r", encoding="utf-8") as fd:
-        content = fd.read()
-        # Make sure we're adding the relevant Glean SDK import, once.
-        assert content.count("import Bar.private.HistogramMetricBase") == 1
-
-        # Validate the generated Gecko metric mapper Kotlin functions.
-        # NOTE: Indentation, whitespaces  and text formatting of the block
-        # below are important. Do not change them unless the file format
-        # changes, otherwise validation will fail.
-        expected_func = """    fun getHistogram(geckoMetricName: String): HistogramMetricBase? {
-        return when (geckoMetricName) {
-            // From GfxContentCheckerboard.kt
-            "CHECKERBOARD_DURATION" -> GfxContentCheckerboard.duration
-            // From PagePerf.kt
-            "GV_PAGE_LOAD_MS" -> PagePerf.loadTime
-            "GV_PAGE_RELOAD_MS" -> PagePerf.reloadTime
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-        expected_func = """    fun getCategoricalMetric(
-        geckoMetricName: String
-    ): LabeledMetricType<CounterMetricType>? {
-        return when (geckoMetricName) {
-            // From PagePerf.kt
-            "DOM_SCRIPT_PRELOAD_RESULT" -> PagePerf.domScriptPreload
-            else -> null
-        }
-    }"""
-
-        assert expected_func in content
-
-        expected_func = """    fun getBooleanScalar(geckoMetricName: String): BooleanMetricType? {
-        return when (geckoMetricName) {
-            // From GfxInfoAdapter.kt
-            "gfx_adapter.stand_alone" -> GfxInfoAdapter.standAlone
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-        expected_func = """    fun getStringScalar(geckoMetricName: String): StringMetricType? {
-        return when (geckoMetricName) {
-            // From GfxInfoAdapter.kt
-            "gfx_adapter.vendor_id" -> GfxInfoAdapter.vendorId
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-        expected_func = """    fun getQuantityScalar(geckoMetricName: String): QuantityMetricType? {
-        return when (geckoMetricName) {
-            // From GfxInfoAdapter.kt
-            "gfx_adapter.width" -> GfxInfoAdapter.screenWidth
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-    for file_name in metrics_files:
-        with (tmpdir / file_name).open("r", encoding="utf-8") as fd:
-            content = fd.read()
-            assert "HistogramMetricBase" not in content
-
-    run_linters(tmpdir.glob("*.kt"))
-
-
-def test_event_extra_keys_in_correct_order(tmpdir):
+def test_event_extra_keys_in_correct_order(tmp_path):
     """
     Assert that the extra keys appear in the parameter and the enumeration in
     the same order.
 
     https://bugzilla.mozilla.org/show_bug.cgi?id=1648768
     """
-
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "event_key_ordering.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo"},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Event.kt", "GleanBuildInfo.kt"]
     )
 
-    with (tmpdir / "Event.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Event.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         content = " ".join(content.split())
         assert "ExampleExtra(" in content
+        assert "and1withextracasing:" in content
         assert "alice:" in content
         assert "bob:" in content
         assert "charlie:" in content
         assert ": EventExtras" in content
-        assert 'allowedExtraKeys = listOf("alice", "bob", "charlie")' in content
+        assert (
+            'allowedExtraKeys = listOf("And1WithExtraCasing", "alice", "bob", "charlie")'
+            in content
+        )
 
 
-def test_arguments_are_generated_in_deterministic_order(tmpdir):
+def test_arguments_are_generated_in_deterministic_order(tmp_path):
     """
     Assert that arguments on generated code are always in the same order.
 
     https://bugzilla.mozilla.org/show_bug.cgi?id=1666192
     """
-
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "event_key_ordering.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo"},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Event.kt", "GleanBuildInfo.kt"]
     )
 
-    with (tmpdir / "Event.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Event.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         content = " ".join(content.split())
-        expected = 'EventMetricType<ExampleExtra> by lazy { // generated from event.example EventMetricType<ExampleExtra>( CommonMetricData( category = "event", name = "example", sendInPings = listOf("events"), lifetime = Lifetime.PING, disabled = false ), allowedExtraKeys = listOf("alice", "bob", "charlie")) } }'  # noqa
+        expected = 'EventMetricType<ExampleExtra> by lazy { // generated from event.example EventMetricType<ExampleExtra>( CommonMetricData( category = "event", name = "example", sendInPings = listOf("events"), lifetime = Lifetime.PING, disabled = false ), allowedExtraKeys = listOf("And1WithExtraCasing", "alice", "bob", "charlie")) } }'  # noqa
         assert expected in content
 
 
-def test_event_extra_keys_with_types(tmpdir):
+def test_event_extra_keys_with_types(tmp_path):
     """
     Assert that the extra keys with types appear with their corresponding types.
     """
-
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "events_with_types.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo"},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Core.kt", "GleanBuildInfo.kt"]
     )
 
-    with (tmpdir / "Core.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Core.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         content = " ".join(content.split())
         assert (
@@ -443,29 +334,29 @@ def test_event_extra_keys_with_types(tmpdir):
         assert (
             'allowedExtraKeys = listOf("enabled", "preference", "swapped")' in content
         )
+        assert "map.put(\"enabled\", it.toString())" in content
+        assert "map.put(\"preference\", it)" in content
+        assert "map.put(\"swapped\", it.toString())" in content
 
 
-def test_reasons(tmpdir):
+def test_reasons(tmp_path):
     """
     Assert that we generate the reason codes as a plain enum.
 
     https://bugzilla.mozilla.org/show_bug.cgi?id=1811888
     """
-
-    tmpdir = Path(str(tmpdir))
-
     translate.translate(
         ROOT / "data" / "pings.yaml",
         "kotlin",
-        tmpdir,
+        tmp_path,
         {"namespace": "Foo"},
     )
 
-    assert set(x.name for x in tmpdir.iterdir()) == set(
+    assert set(x.name for x in tmp_path.iterdir()) == set(
         ["Pings.kt", "GleanBuildInfo.kt"]
     )
 
-    with (tmpdir / "Pings.kt").open("r", encoding="utf-8") as fd:
+    with (tmp_path / "Pings.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         content = " ".join(content.split())
 
@@ -477,3 +368,42 @@ def test_reasons(tmpdir):
 
     expected = "val customPingMightBeEmpty: PingType<customPingMightBeEmptyReasonCodes> = // generated from custom-ping-might-be-empty"  # noqa
     assert expected in content
+
+
+def test_object_metric(tmp_path):
+    """
+    Assert that an object metric is created.
+    """
+    translate.translate(
+        ROOT / "data" / "object.yaml",
+        "kotlin",
+        tmp_path,
+        {"namespace": "Foo"},
+    )
+
+    assert set(x.name for x in tmp_path.iterdir()) == set(
+        ["ActivityStream.kt", "ComplexTypes.kt", "CrashStack.kt", "GleanBuildInfo.kt"]
+    )
+
+    with (tmp_path / "ComplexTypes.kt").open("r", encoding="utf-8") as fd:
+        content = fd.read()
+        content = " ".join(content.split())
+
+        assert "typealias ArrayInArrayObjectItemItem = Boolean" in content
+        assert "typealias NumberArrayObjectItem = Int" in content
+
+    with (tmp_path / "CrashStack.kt").open("r", encoding="utf-8") as fd:
+        content = fd.read()
+        content = " ".join(content.split())
+
+        assert "ObjectMetricType<ThreadsObject>" in content
+        assert "data class ThreadsObject(" in content
+        assert "data class ThreadsObjectItem(" in content
+        assert (
+            "var frames: ThreadsObjectItemFrames = ThreadsObjectItemFrames" in content
+        )
+
+        assert "data class ThreadsObjectItemFramesItem(" in content
+        assert "var moduleIndex: Int? = null," in content
+        assert "var ip: String? = null," in content
+        assert "var trust: String? = null," in content
