@@ -136,7 +136,7 @@ def test_combined():
                 "m_width_pixels": {
                     "type": "quantity",
                     "gecko_datapoint": "WIDTH_PIXELS",
-                    "unit": "pixels", # not a lint issue
+                    "unit": "pixels",  # not a lint issue
                 },
             }
         }
@@ -525,6 +525,53 @@ def test_name_too_similar_lint():
     assert "all_metrics.valid_metric" in nits[0].msg
 
 
+def test_redefined_metric():
+    """Ensure we fail when folks redefine a metric with the same category+name."""
+    input = [
+        ROOT / "data" / "redefined_metric.yamlx",
+        ROOT / "data" / "same_name_different_category.yaml",
+    ]
+    all_objects = parser.parse_objects(input)
+
+    errs = list(all_objects)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_objects.value, parser_config={})
+    assert len(nits) == 1
+    assert nits[0].check_name == "REDEFINED_METRIC"
+    assert nits[0].name == "redefined.metric"
+    assert "metric_name" in nits[0].msg
+
+
+def test_redefined_category():
+    """Ensure we fail when folks redefine a category."""
+    input = [
+        ROOT / "data" / "redefined_category.yamlx",
+        ROOT / "data" / "same_name_different_category.yaml",
+    ]
+    all_objects = parser.parse_objects(input)
+
+    errs = list(all_objects)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_objects.value, parser_config={})
+    assert len(nits) == 1
+    assert nits[0].check_name == "REDEFINED_CATEGORY"
+
+
+def test_redefined_ping():
+    """Ensure we fail when folks redefine a ping."""
+    input = [
+        ROOT / "data" / "redefined_ping.yamlx",
+        ROOT / "data" / "pings.yaml",
+    ]
+    all_objects = parser.parse_objects(input)
+
+    errs = list(all_objects)
+    assert len(errs) == 1
+    assert "Duplicate ping named 'some-ping'" in errs[0]
+
+
 @pytest.mark.parametrize(
     "metric, num_nits",
     [
@@ -534,6 +581,7 @@ def test_name_too_similar_lint():
         ({"metric": {"data_reviews": [""], "no_lint": ["EMPTY_DATAREVIEW"]}}, 0),
         ({"metric": {"data_reviews": ["TODO"]}}, 1),
         ({"metric": {"data_reviews": ["TODO"], "no_lint": ["EMPTY_DATAREVIEW"]}}, 0),
+        ({"metric": {"data_reviews": ["TBD"]}}, 1),
     ],
 )
 def test_empty_datareviews(metric, num_nits):
@@ -597,3 +645,233 @@ def test_unit_on_metrics(metric, num_nits):
     assert len(nits) == num_nits
     if num_nits > 0:
         assert set(["UNEXPECTED_UNIT"]) == set(v.check_name for v in nits)
+
+
+@pytest.mark.parametrize(
+    "metric, num_nits",
+    [
+        (
+            {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {
+                        "id": {"type": "string", "description": "description"}
+                    },
+                }
+            },
+            0,
+        ),
+        (
+            {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {
+                        "url": {"type": "string", "description": "description"}
+                    },
+                }
+            },
+            1,
+        ),
+        (
+            {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {
+                        "url": {"type": "string", "description": "description"}
+                    },
+                    "no_lint": ["HIGHER_DATA_SENSITIVITY_REQUIRED"],
+                }
+            },
+            0,
+        ),
+        (
+            {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {
+                        "url": {"type": "string", "description": "description"}
+                    },
+                    "data_sensitivity": ["technical"],
+                }
+            },
+            1,
+        ),
+        (
+            {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {
+                        "url": {"type": "string", "description": "description"}
+                    },
+                    "data_sensitivity": ["highly_sensitive"],
+                }
+            },
+            0,
+        ),
+        (
+            {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {
+                        "url": {"type": "string", "description": "description"}
+                    },
+                    "data_sensitivity": ["technical", "highly_sensitive"],
+                }
+            },
+            0,
+        ),
+    ],
+)
+def test_events_data_sensitivity(metric, num_nits):
+    content = {"category": metric}
+    content = util.add_required(content)
+    all_metrics = parser.parse_objects(content)
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_metrics.value)
+
+    assert len(nits) == num_nits
+    if num_nits > 0:
+        assert set(["HIGHER_DATA_SENSITIVITY_REQUIRED"]) == set(
+            v.check_name for v in nits
+        )
+
+
+def test_events_data_sensitivity_from_file():
+    """Test that the 'glinter' reports issues with the old event API."""
+    all_metrics = parser.parse_objects([ROOT / "data" / "events_data_sensitivity.yaml"])
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_metrics.value, parser_config={})
+    assert len(nits) == 2
+    assert nits[0].check_name == "HIGHER_DATA_SENSITIVITY_REQUIRED"
+    assert nits[0].name == "event.low_sensitivity"
+    assert "data sensitivity" in nits[0].msg
+
+    assert nits[1].check_name == "HIGHER_DATA_SENSITIVITY_REQUIRED"
+    assert nits[1].name == "event.no_data_sensitivity"
+    assert "data sensitivity" in nits[1].msg
+
+
+@pytest.mark.parametrize(
+    "content,num_nits",
+    [
+        (
+            {
+                "valid_expire_lint": {
+                    "type": "counter",
+                    "expires": "2100-01-01",
+                    "no_lint": ["EXPIRATION_DATE_TOO_FAR"],
+                }
+            },
+            0,
+        ),
+        (
+            {
+                "too_many_no_lints": {
+                    "type": "counter",
+                    "expires": "2100-01-01",
+                    "no_lint": ["EXPIRATION_DATE_TOO_FAR", "UNIT_IN_NAME", "UNUSED_NO_LINT"],
+                }
+            },
+            0,
+        ),
+        (
+            {
+                "too_many_no_lints": {
+                    "type": "counter",
+                    "expires": "2100-01-01",
+                    "no_lint": ["EXPIRATION_DATE_TOO_FAR", "UNIT_IN_NAME"],
+                }
+            },
+            1,
+        ),
+        (
+            {
+                "not_applying_no_lint": {
+                    "type": "counter",
+                    "expires": "never",
+                    "no_lint": ["EXPIRATION_DATE_TOO_FAR"],
+                }
+            },
+            1,
+        ),
+    ],
+)
+def test_unused_no_lint_warning(content, num_nits):
+    content = {"cat": content}
+    content = util.add_required(content)
+    all_metrics = parser.parse_objects(content)
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_metrics.value)
+    assert len(nits) == num_nits
+    if num_nits > 0:
+        assert set(["UNUSED_NO_LINT"]) == set(
+            v.check_name for v in nits
+        )
+
+@pytest.mark.parametrize(
+    "content,num_nits",
+    [
+        (
+            {
+                "unknown_lint": {
+                    "type": "counter",
+                    "expires": "2100-01-01",
+                    "no_lint": ["EXPIRATION_DATE_TOO_FAR", "DOES_NOT_EXIST", "UNKNOWN_LINT"],
+                }
+            },
+            0,
+        ),
+        (
+            {
+                "unknown_lint": {
+                    "type": "counter",
+                    "expires": "2100-01-01",
+                    "no_lint": ["EXPIRATION_DATE_TOO_FAR", "DOES_NOT_EXIST"],
+                }
+            },
+            1,
+        ),
+        (
+            {
+                "unknown_lint": {
+                    "type": "counter",
+                    "expires": "never",
+                    "no_lint": ["DOES_NOT_EXIST"],
+                }
+            },
+            1,
+        ),
+        (
+            {
+                "ping_lint": {
+                    "type": "counter",
+                    "expires": "never",
+                    "no_lint": ["REDUNDANT_PING"],
+                }
+            },
+            1,
+        ),
+    ],
+)
+def test_unknown_lint_warning(content, num_nits):
+    content = {"cat": content}
+    content = util.add_required(content)
+    all_metrics = parser.parse_objects(content)
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_metrics.value)
+    assert len(nits) == num_nits
+    if num_nits > 0:
+        assert set(["UNKNOWN_LINT"]) == set(
+            v.check_name for v in nits
+        )
