@@ -479,7 +479,7 @@ def invalid_in_label(name):
 @pytest.mark.parametrize(
     "name",
     [
-        "1" * 72,
+        "1" * 112,
         "Møøse",
     ],
 )
@@ -1153,7 +1153,14 @@ def test_no_internal_fields_exposed():
                             "extra_keys": {
                                 "key_a": {"description": "desc-a", "type": "boolean"}
                             },
-                        }
+                        },
+                        "metric2": {
+                            "type": "dual_labeled_counter",
+                            "dual_labels": {
+                                "key": {"description": "desc-a"},
+                                "category": {"description": "desc-b"},
+                            },
+                        },
                     },
                 }
             ),
@@ -1185,7 +1192,25 @@ def test_no_internal_fields_exposed():
             "send_in_pings": ["events"],
             "type": "event",
             "version": 0,
-        }
+        },
+        "category.metric2": {
+            "bugs": ["http://bugzilla.mozilla.org/12345678"],
+            "categories": None,
+            "data_reviews": ["https://example.com/review/"],
+            "defined_in": {"line": 17},
+            "description": "DESCRIPTION...",
+            "disabled": False,
+            "expires": "never",
+            "gecko_datapoint": "",
+            "keys": None,
+            "lifetime": "ping",
+            "metadata": {},
+            "no_lint": [],
+            "notification_emails": ["nobody@example.com"],
+            "send_in_pings": ["metrics"],
+            "type": "dual_labeled_counter",
+            "version": 0,
+        },
     }
     expected_json = json.dumps(expected, sort_keys=True, indent=2)
 
@@ -1261,6 +1286,130 @@ def test_object_invalid():
     assert "invalid `type` in object structure." in errors[0]
 
 
+@pytest.mark.parametrize(
+    "structure,num_errors",
+    [
+        (
+            {
+                "type": "object",
+                "properties": {"key": {"type": "string"}, "value": {"oneOf": []}},
+            },
+            1,
+        ),
+        (
+            {
+                "type": "object",
+                "properties": {"key": {"type": "string"}, "value": {"oneOf": [ { "type": "object" } ]}},
+            },
+            1,
+        ),
+        (
+            {
+                "type": "object",
+                "properties": {"key": {"type": "string"}, "value": {"oneOf": [ { "type": "number", "other": "unsupported" } ]}},
+            },
+            1,
+        ),
+        (
+            {
+                "type": "object",
+                "properties": {"key": {"type": "string"}, "value": {"oneOf": [ { "type": "number" } ]}},
+            },
+            0,
+        ),
+        (
+            {
+                "type": "object",
+                "properties": {"key": {"type": "string"}, "value": {"oneOf": [ { "type": "number", "description": "desc" } ]}},
+            },
+            0,
+        ),
+    ],
+)
+def test_object_oneof(structure, num_errors):
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == num_errors, errors
+    assert len(all_metrics.value) == 1
+    # assert all_metrics.value["category"]["metric"]._generate_structure == structure
+
+
+def test_dual_labeled_counter():
+    """
+    Ensure that `dual_labeled_counter` metrics parse properly.
+    """
+
+    all_metrics = parser.parse_objects(
+        [ROOT / "data" / "dual_labeled.yaml"],
+        config={"allow_reserved": False},
+    )
+
+    errors = list(all_metrics)
+    assert len(errors) == 0
+
+    assert all_metrics.value["test.dual_labeled"]["static_static"].ordered_keys == [
+        "key1",
+        "key2",
+    ]
+    assert all_metrics.value["test.dual_labeled"][
+        "static_static"
+    ].ordered_categories == ["category1", "category2"]
+    assert all_metrics.value["test.dual_labeled"]["static_dynamic"].ordered_keys == [
+        "key1",
+        "key2",
+    ]
+    assert (
+        all_metrics.value["test.dual_labeled"]["static_dynamic"].ordered_categories
+        is None
+    )
+    assert all_metrics.value["test.dual_labeled"]["dynamic_static"].ordered_keys is None
+    assert all_metrics.value["test.dual_labeled"][
+        "dynamic_static"
+    ].ordered_categories == ["category1", "category2"]
+    assert (
+        all_metrics.value["test.dual_labeled"]["dynamic_dynamic"].ordered_keys is None
+    )
+    assert (
+        all_metrics.value["test.dual_labeled"]["dynamic_dynamic"].ordered_categories
+        is None
+    )
+
+
+def test_dual_labeled_counter_invalid():
+    """
+    Ensure that invalidly-defined `dual_labeled_counter` metrics fail to parse.
+    """
+
+    all_metrics = parser.parse_objects(
+        [ROOT / "data" / "dual_labeled_invalid.yaml"],
+        config={"allow_reserved": False},
+    )
+
+    errors = list(all_metrics)
+    assert len(errors) == 4
+
+    def compare(expected, found):
+        return "".join(expected.split()) in "".join(found.split())
+
+    for error in errors:
+        if "`key`" in error:
+            assert compare("`dual_labels` is missing required parameter `key`", error)
+
+        if "`category`" in error:
+            assert compare(
+                "`dual_labels` is missing required parameter `categories`", error
+            )
+
+        if "`dual_labels`" in error:
+            assert compare(
+                "`dual_labeled_counter` is missing required parameter `dual_labels`",
+                error,
+            )
+
+
 def test_reserved_category():
     all_metrics = parser.parse_objects(ROOT / "data" / "reserved_categories.yamlx")
 
