@@ -132,7 +132,7 @@ def test_metric_type_name():
         expires="never",
     )
 
-    assert rust.type_name(event) == "EventMetric<NoExtra>"
+    assert rust.type_name(event) == "EventMetric<NoExtraKeys>"
 
     boolean = metrics.Boolean(
         type="boolean",
@@ -287,3 +287,51 @@ def test_object_metric(tmp_path):
         assert "ip: Option<String>, " in content
         assert "trust: Option<String>, " in content
         assert "}" in content
+
+
+def test_dual_labeled_counter_metric(tmp_path):
+    """
+    Assert that a dual labeled counter metric is created.
+    """
+    translate.translate(
+        ROOT / "data" / "dual_labeled.yaml",
+        "rust",
+        tmp_path,
+        {"namespace": "Foo"},
+    )
+
+    assert set(x.name for x in tmp_path.iterdir()) == set(["glean_metrics.rs"])
+
+    with (tmp_path / "glean_metrics.rs").open("r", encoding="utf-8") as fd:
+        content = fd.read()
+        content = " ".join(content.split())
+
+        assert (
+            "pub static dynamic_dynamic: ::glean::private::__export::Lazy<DualLabeledCounterMetric>"
+            in content
+        )
+        assert "DualLabeledCounterMetric::new(meta, None, None)" in content
+        assert (
+            "pub static static_dynamic: ::glean::private::__export::Lazy<DualLabeledCounterMetric>"
+            in content
+        )
+        assert (
+            'DualLabeledCounterMetric::new(meta, Some(vec![::std::borrow::Cow::from("key1"), ::std::borrow::Cow::from("key2")]), None)'
+            in content
+        )
+        assert (
+            "pub static dynamic_static: ::glean::private::__export::Lazy<DualLabeledCounterMetric>"
+            in content
+        )
+        assert (
+            'DualLabeledCounterMetric::new(meta, None, Some(vec![::std::borrow::Cow::from("category1"), ::std::borrow::Cow::from("category2")]))'
+            in content
+        )
+        assert (
+            "pub static static_static: ::glean::private::__export::Lazy<DualLabeledCounterMetric>"
+            in content
+        )
+        assert (
+            'DualLabeledCounterMetric::new(meta, Some(vec![::std::borrow::Cow::from("key1"), ::std::borrow::Cow::from("key2")]), Some(vec![::std::borrow::Cow::from("category1"), ::std::borrow::Cow::from("category2")]))'
+            in content
+        )
