@@ -288,9 +288,10 @@ def test_event_extra_keys_in_correct_order(tmp_path):
         content = " ".join(content.split())
         assert (
             "struct ExampleExtra: EventExtras "
-            "{ var alice: String? var bob: String? var charlie: String?" in content
+            "{ var and1withextracasing: Bool? var alice: String? var bob: String? var charlie: String?"
+            in content
         )
-        assert ', ["alice", "bob", "charlie"]' in content
+        assert ', ["And1WithExtraCasing", "alice", "bob", "charlie"]' in content
 
 
 def test_event_extra_keys_with_types(tmp_path):
@@ -336,7 +337,7 @@ def test_reasons(tmp_path):
         content = fd.read()
         content = " ".join(content.split())
 
-    expected = "enum CustomPingMightBeEmptyReasonCodes: Int, ReasonCodes { case serious = 0 case silly = 1 public func index() -> Int { return self.rawValue } }"  # noqa
+    expected = "enum CustomPingMightBeEmptyReasonCodes: Int, ReasonCodes, Sendable { case serious = 0 case silly = 1 public func index() -> Int { return self.rawValue } }"  # noqa
     assert expected in content
 
     expected = "let customPing = Ping<NoReasonCodes>("
@@ -344,3 +345,39 @@ def test_reasons(tmp_path):
 
     expected = "let customPingMightBeEmpty = Ping<CustomPingMightBeEmptyReasonCodes>("
     assert expected in content
+
+
+def test_object_metric(tmp_path):
+    """
+    Assert that an object metric is created.
+    """
+    translate.translate(
+        ROOT / "data" / "object.yaml",
+        "swift",
+        tmp_path,
+        {"namespace": "Foo"},
+    )
+
+    assert set(x.name for x in tmp_path.iterdir()) == set(["Metrics.swift"])
+
+    with (tmp_path / "Metrics.swift").open("r", encoding="utf-8") as fd:
+        content = fd.read()
+        content = " ".join(content.split())
+
+        assert "typealias ArrayInArrayObjectItemItem = Bool" in content
+        assert "typealias NumberArrayObjectItem = Int64" in content
+
+        assert "ObjectMetricType<ThreadsObject>" in content
+        assert "typealias ThreadsObject = [ThreadsObjectItem]" in content
+        assert (
+            "struct ThreadsObjectItem: Codable, Equatable {" in content
+        )
+        assert "var frames: ThreadsObjectItemFrames" in content
+
+        assert (
+            "struct ThreadsObjectItemFramesItem: Codable, Equatable {"
+            in content
+        )
+        assert "var moduleIndex: Int64?" in content
+        assert "var ip: String?" in content
+        assert "var trust: String?" in content
